@@ -11,6 +11,12 @@
 
 PXR_NAMESPACE_OPEN_SCOPE
 
+TraceEventNodeRefPtr
+TraceEventNode::New() {
+    return TraceEventNode::New(
+        TfToken("root"), TraceCategory::Default, 0, 0, {}, false);
+}
+
 TraceEventNodeRefPtr
 TraceEventNode::Append(
     const TfToken &key, 
@@ -22,14 +28,14 @@ TraceEventNode::Append(
     TraceEventNodeRefPtr n = 
         TraceEventNode::New(
             key, category, beginTime, endTime, {}, separateEvents);
-    _children.push_back(n);
+    Append(n);
     return n;
 }
 
 void
 TraceEventNode::Append(TraceEventNodeRefPtr node)
 {
-    _children.push_back(node);
+    _children.emplace_back(std::move(node));
 }
 
 void 
@@ -48,14 +54,33 @@ TraceEventNode::SetBeginAndEndTimesFromChildren()
         _beginTime = std::min(_beginTime, c->GetBeginTime());
         _endTime   = std::max(_endTime, c->GetEndTime());
     }
+}
 
+const TraceEventNode::AttributeMap&
+TraceEventNode::GetAttributes() const
+{
+    static const AttributeMap empty;
+    if (AttributeMap const *attrMap = _attributesAndSeparateEvents.Get()) {
+        return *attrMap;
+    }
+    return empty;
 }
 
 void
 TraceEventNode::AddAttribute(
-    const TfToken& key, const AttributeData& attr)
+    const TfToken& key, AttributeData&& attr)
 {
-    _attributes.emplace(key, attr);
+    if (!_attributesAndSeparateEvents.Get()) {
+        _attributesAndSeparateEvents.Set(new AttributeMap);
+    }
+    _attributesAndSeparateEvents->emplace(key, std::move(attr));
 }
 
-PXR_NAMESPACE_CLOSE_SCOPE
\ No newline at end of file
+void
+TraceEventNode::_DeleteAttrMap(AttributeMap *attrMap)
+{
+    delete attrMap;
+}
+
+
+PXR_NAMESPACE_CLOSE_SCOPE
