@@ -275,6 +275,7 @@ private:
             , arcType(PcpArcTypeRoot)
             , permission(SdfPermissionPublic)
             , hasSymmetry(false)
+            , hasValueClips(false)
             , inert(false)
             , permissionDenied(false)
         */
@@ -352,6 +353,11 @@ private:
             // or at any of its namespace ancestors contain symmetry 
             // information.
             bool hasSymmetry:1;
+            // Whether this node may contribute value clips information
+            // during composition. This implies that prims at this node's
+            // site or at any of its namespace ancestors contain value clips
+            // information.
+            bool hasValueClips:1;
             // Whether this node is inert. This is set to true in cases
             // where a node is needed to represent a structural dependency
             // but no opinions are allowed to be added.
