@@ -11,6 +11,7 @@
 #include "pxr/base/tf/type.h"
 
 #include "pxr/base/vt/array.h"
+#include "pxr/base/vt/arrayEdit.h"
 #include "pxr/base/vt/value.h"
 
 #include <ostream>
@@ -23,6 +24,7 @@ TF_REGISTRY_FUNCTION(TfType)
 {
     TfType::Define<SdfTimeCode>();
     TfType::Define< VtArray<SdfTimeCode> >();
+    TfType::Define< VtArrayEdit<SdfTimeCode> >();
 }
 
 TF_REGISTRY_FUNCTION(VtValue)
