@@ -6,6 +6,7 @@
 //
 #include "pxr/pxr.h"
 #include "pxr/usd/usd/attribute.h"
+#include "pxr/usd/usd/attributeLimits.h"
 #include "pxr/usd/usd/attributeQuery.h"
 #include "pxr/usd/usd/common.h"
 #include "pxr/usd/usd/instanceCache.h"
@@ -31,7 +32,6 @@
 
 PXR_NAMESPACE_OPEN_SCOPE
 
-
 // ------------------------------------------------------------------------- //
 // UsdAttribute 
 // ------------------------------------------------------------------------- //
@@ -75,6 +75,13 @@ UsdAttribute::Block() const
     Set(VtValue(SdfValueBlock()), UsdTimeCode::Default()); 
 }
 
+void
+UsdAttribute::BlockAnimation() const
+{
+    Clear();
+    Set(VtValue(SdfAnimationBlock()), UsdTimeCode::Default());
+}
+
 bool
 UsdAttribute::GetTimeSamples(std::vector<double>* times) const 
 {
@@ -238,31 +245,42 @@ UsdAttribute::Set(const char* value, UsdTimeCode time) const {
 
 bool 
 UsdAttribute::Set(const VtValue& value, UsdTimeCode time) const 
-{ 
+{
+    if (value.IsHolding<SdfAnimationBlock>() && 
+        time != UsdTimeCode::Default()) {
+        TF_CODING_ERROR("Cannot set SdfAnimationBlock on <%s> at time %g. "
+                        "Animation blocks can only be set at the default time.",
+                        GetPath().GetText(), time.GetValue());
+        return false;
+    }
     return _GetStage()->_SetValue(time, *this, value);
 }
 
 bool
 UsdAttribute::HasSpline() const
 {
-    return _GetStage()->_HasMetadata(
-        *this,                 // find a field in our attribute spec
-        SdfFieldKeys->Spline,  // find the Spline field
-        TfToken(),             // not a dict field, so no dict key
-        false);                // want authored opinions only
+    UsdResolveInfo resolveInfo = GetResolveInfo();
+    return resolveInfo.GetSource() == UsdResolveInfoSourceSpline;
 }
 
 TsSpline
 UsdAttribute::GetSpline() const
 {
-    TsSpline spline;
-    _GetStage()->_GetMetadata(
-        *this,                 // read a field in our attribute spec
-        SdfFieldKeys->Spline,  // read the Spline field
-        TfToken(),             // not a dict field, so no dict key
-        false,                 // want authored opinions only
-        &spline);              // read into this variable
-    return spline;
+    UsdResolveInfo resolveInfo = GetResolveInfo();
+    if (resolveInfo.GetSource() == UsdResolveInfoSourceSpline) {
+        // Don't return resolveInfo._spline directly because we need
+        // to compute layer offsets
+        TsSpline spline;
+        _GetStage()->_GetMetadata(
+            *this,                 // read a field in our attribute spec
+            SdfFieldKeys->Spline,  // read the Spline field
+            TfToken(),             // not a dict field, so no dict key
+            false,                 // want authored opinions only
+            &spline);              // read into this variable
+        return spline;
+    } else {
+        return TsSpline();
+    }
 }
 
 bool
@@ -321,6 +339,106 @@ UsdAttribute::ClearColorSpace() const
     return ClearMetadata(SdfFieldKeys->ColorSpace);
 }
 
+VtDictionary
+UsdAttribute::GetLimits() const
+{
+    VtDictionary limits;
+    if (GetMetadata(SdfFieldKeys->Limits, &limits)) {
+        return limits;
+    }
+    return {};
+}
+
+bool
+UsdAttribute::SetLimits(const VtDictionary& limits) const
+{
+    VtDictionary conformedLimits;
+    bool isValid = true;
+
+    // Verify `limits` contains only valid sub-dicts
+    for (const auto& it : limits) {
+        const std::string& key = it.first;
+        const VtValue& value = it.second;
+
+        if (!value.IsHolding<VtDictionary>()) {
+            TF_CODING_ERROR(
+                "Cannot set limits dictionary for <%s> (must contain only "
+                "sub-dictionary entries)",
+                GetPath().GetText());
+            return false;
+        }
+
+        const VtDictionary& subDict = value.UncheckedGet<VtDictionary>();
+        UsdAttributeLimits::ValidationResult result;
+
+        if (GetLimits(TfToken(key)).Validate(subDict, &result)) {
+            conformedLimits[key] = result.GetConformedSubDict();
+        }
+        else {
+            TF_CODING_ERROR(result.GetErrorString());
+            isValid = false;
+        }
+    }
+
+    return isValid && SetMetadata(SdfFieldKeys->Limits, conformedLimits);
+}
+
+bool
+UsdAttribute::HasAuthoredLimits() const
+{
+    return HasAuthoredMetadata(SdfFieldKeys->Limits);
+}
+
+bool
+UsdAttribute::ClearLimits() const
+{
+    return ClearMetadata(SdfFieldKeys->Limits);
+}
+
+UsdAttributeLimits
+UsdAttribute::GetSoftLimits() const
+{
+    return UsdAttributeLimits(*this, UsdLimitsKeys->Soft);
+}
+
+UsdAttributeLimits
+UsdAttribute::GetHardLimits() const
+{
+    return UsdAttributeLimits(*this, UsdLimitsKeys->Hard);
+}
+
+UsdAttributeLimits
+UsdAttribute::GetLimits(const TfToken& key) const
+{
+    return UsdAttributeLimits(*this, key);
+}
+
+int64_t
+UsdAttribute::GetArraySizeConstraint() const
+{
+    int64_t constraint = 0;
+    GetMetadata(SdfFieldKeys->ArraySizeConstraint, &constraint);
+    return constraint;
+}
+
+bool
+UsdAttribute::SetArraySizeConstraint(int64_t constraint) const
+{
+    return SetMetadata(SdfFieldKeys->ArraySizeConstraint, constraint);
+}
+
+bool
+UsdAttribute::HasAuthoredArraySizeConstraint() const
+{
+    return HasMetadata(SdfFieldKeys->ArraySizeConstraint);
+}
+
+bool 
+UsdAttribute::ClearArraySizeConstraint() const
+{
+    return ClearMetadata(SdfFieldKeys->ArraySizeConstraint);
+}
+
 SdfAttributeSpecHandle
 UsdAttribute::_CreateSpec(const SdfValueTypeName& typeName, bool custom,
                           const SdfVariability &variability) const
@@ -379,10 +497,12 @@ ARCH_PRAGMA_INSTANTIATION_AFTER_SPECIALIZATION
 TF_PP_SEQ_FOR_EACH(_INSTANTIATE_GET, ~, SDF_VALUE_TYPES)
 #undef _INSTANTIATE_GET
 
-// In addition to the Sdf value types, _Set can also be called with an 
-// SdfValueBlock.
+// In addition to the Sdf value types, _Set can also be called with 
+// SdfValueBlock or SdfAnimationBlock.
 template USD_API bool UsdAttribute::_Set(
     const SdfValueBlock &, UsdTimeCode) const;
+template USD_API bool UsdAttribute::_Set(
+    const SdfAnimationBlock &, UsdTimeCode) const;
 
 ARCH_PRAGMA_POP
 
