@@ -24,6 +24,10 @@ TF_DEFINE_PUBLIC_TOKENS(
     UsdSkelImagingExtComputationInputNameTokens,
     USD_SKEL_IMAGING_EXT_COMPUTATION_INPUT_NAME_TOKENS);
 
+TF_DEFINE_PUBLIC_TOKENS(
+    UsdSkelImagingExtComputationLegacyInputNameTokens,
+    USD_SKEL_IMAGING_EXT_COMPUTATION_LEGACY_INPUT_NAME_TOKENS);
+
 TF_DEFINE_PUBLIC_TOKENS(
     UsdSkelImagingExtComputationOutputNameTokens,
     USD_SKEL_IMAGING_EXT_COMPUTATION_OUTPUT_NAME_TOKENS);
